<?php

namespace Drupal\form_display_field_alter\Entity;

use Drupal\Core\Entity\Entity\EntityFormDisplay as CoreEntityFormDisplay;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldConfigInterface;
use Drupal\Core\Field\FieldDefinition;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\WidgetInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a class used to override Drupal core's entity form display class.
 *
 * Copyright (C) 2025  Library Solutions, LLC (et al.).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * @internal
 */
final class EntityFormDisplay extends CoreEntityFormDisplay {

  /**
   * Conditionally alter the supplied field definition.
   *
   * This method duplicates configurable field definitions, then invokes a hook
   * which allows them to be altered before building the form. All other field
   * definitions remain untouched.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $definition
   *   The field definition to alter if applicable.
   *
   * @see \hook_entity_form_display_field_definition_alter()
   *   For more information about how to alter field definitions before being
   *   used when rendering a form display.
   *
   * @return \Drupal\Core\Field\FieldDefinitionInterface
   *   The field definition after being altered if applicable.
   */
  private function alterFieldDefinition(FieldDefinitionInterface $definition): FieldDefinitionInterface {
    if ($definition instanceof BaseFieldDefinition || $definition instanceof FieldConfigInterface || $definition instanceof FieldDefinition) {
      $definition = \_form_display_field_alter_create_duplicate_field_definition($definition);
      $context = [
        'form_mode' => $this->mode ?? static::CUSTOM_MODE,
        'third_party_settings' => $this->content[$definition->getName()]['third_party_settings'] ?? [],
      ];

      $module_handler = $this->moduleHandler();
      $module_handler->alter('entity_form_display_field_definition', $definition, $context);
    }

    return $definition;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(FieldableEntityInterface $entity, array &$form, FormStateInterface $form_state) {
    $form['#cache']['max-age'] = 0;

    $form['#parents'] ??= [];
    $form['#process'][] = [$this, 'processForm'];

    foreach ($this->getComponents() as $name => $options) {
      $widget = $this->getRenderer($name);

      if ($widget instanceof WidgetInterface) {
        $items = $entity->get($name);
        $items->filterEmptyItems();

        // Recreate the field item list to use its altered definition.
        $items = \_form_display_field_alter_recreate_field_item_list($items, $this->getFieldDefinition($name));

        $form[$name] = $widget->form($items, $form, $form_state);
        $form[$name]['#access'] = $items->access('edit');
        $form[$name]['#weight'] = $options['weight'];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getFieldDefinitions() {
    if (!isset($this->fieldDefinitions)) {
      $entity_field_manager = \Drupal::service('entity_field.manager');
      $definitions = $entity_field_manager->getFieldDefinitions($this->targetEntityType, $this->bundle);

      if ($this->mode !== static::CUSTOM_MODE) {
        $definitions = \array_filter($definitions, $this->isCandidateForDisplay(...));
      }

      $this->fieldDefinitions = \array_map($this->alterFieldDefinition(...), $definitions);
    }

    return $this->fieldDefinitions;
  }

  /**
   * Determines if a field definition is a candidate for display.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $definition
   *   The field definition to check.
   *
   * @return bool
   *   TRUE if the field definition is a candidate for display, FALSE otherwise.
   */
  private function isCandidateForDisplay(FieldDefinitionInterface $definition): bool {
    return !empty($definition->getDisplayOptions($this->displayContext));
  }

}
