<?php

namespace Drupal\Tests\form_display_field_alter\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the 'form_display_field_settings' module.
 *
 * Copyright (C) 2025  Library Solutions, LLC (et al.).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * @covers \form_display_field_settings_entity_form_display_field_definition_alter()
 * @covers \form_display_field_settings_field_widget_third_party_settings_form()
 * @group form_display_field_alter
 */
class FormDisplayFieldSettingsTest extends BrowserTestBase {

  /**
   * The content type to use for testing.
   *
   * @var \Drupal\node\NodeTypeInterface
   */
  protected $contentType;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field_ui',
    'form_display_field_settings',
    'node',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->contentType = $this->createContentType();
    $this->contentType->save();

    $this->drupalLogin($this->drupalCreateUser([
      'administer node form display',
      "create {$this->contentType->id()} content",
    ]));
  }

  /**
   * Test the form display field settings.
   */
  public function testFormDisplayFieldSettings(): void {
    $assert_session = $this->assertSession();

    $this->drupalGet("admin/structure/types/manage/{$this->contentType->id()}/form-display");
    $this->submitForm([], 'title_settings_edit');

    // Ensure that a required field has label, description, and default value
    // fields, but not a required field.
    $prefix = 'fields[title][settings_edit_form][third_party_settings][form_display_field_settings]';
    $assert_session->fieldValueEquals("{$prefix}[label]", '');
    $assert_session->fieldValueEquals("{$prefix}[description]", '');
    $assert_session->fieldNotExists("{$prefix}[required]");
    $assert_session->fieldValueEquals("{$prefix}[default_value][title][0][value]", '');

    // Ensure that the entered values are stored.
    $edit["{$prefix}[label]"] = $this->randomMachineName();
    $edit["{$prefix}[description]"] = $this->randomMachineName();
    $edit["{$prefix}[default_value][title][0][value]"] = $this->randomMachineName();

    $this->submitForm($edit, 'Update');
    $this->submitForm($edit = [], 'Save');

    $this->submitForm([], 'title_settings_edit');

    $assert_session->fieldValueNotEquals("{$prefix}[label]", '');
    $assert_session->fieldValueNotEquals("{$prefix}[description]", '');
    $assert_session->fieldValueNotEquals("{$prefix}[default_value][title][0][value]", '');

    // Ensure that an optional field has label, description, required, and
    // default value fields.
    $this->submitForm([], 'body_settings_edit');

    $prefix = 'fields[body][settings_edit_form][third_party_settings][form_display_field_settings]';
    $assert_session->fieldValueEquals("{$prefix}[label]", '');
    $assert_session->fieldValueEquals("{$prefix}[description]", '');
    $assert_session->fieldValueEquals("{$prefix}[required]", '');
    $assert_session->fieldValueEquals("{$prefix}[default_value][body][0][value]", '');

    // Ensure that the entered values are stored.
    $edit["{$prefix}[label]"] = $this->randomMachineName();
    $edit["{$prefix}[description]"] = $this->randomMachineName();
    $edit["{$prefix}[required]"] = '1';
    $edit["{$prefix}[default_value][body][0][value]"] = $this->randomMachineName();

    $this->submitForm($edit, 'Update');
    $this->submitForm($edit = [], 'Save');

    $this->submitForm([], 'body_settings_edit');

    $assert_session->fieldValueNotEquals("{$prefix}[label]", '');
    $assert_session->fieldValueNotEquals("{$prefix}[description]", '');
    $assert_session->fieldValueEquals("{$prefix}[required]", '1');
    $assert_session->fieldValueNotEquals("{$prefix}[default_value][body][0][value]", '');

    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface */
    $entity_display_repository = \Drupal::service('entity_display.repository');

    $display = $entity_display_repository->getFormDisplay('node', $this->contentType->id());
    $components = [
      'title' => $display->getComponent('title'),
      'body' => $display->getComponent('body'),
    ];

    foreach ($components as $name => $component) {
      $component = $component['third_party_settings']['form_display_field_settings'] ?? [];

      $this->assertArrayHasKey($key = 'field_type', $component, "{$name} has third party setting {$key}");
      $this->assertArrayHasKey($key = 'label', $component, "{$name} has third party setting {$key}");
      $this->assertArrayHasKey($key = 'description', $component, "{$name} has third party setting {$key}");
      $this->assertArrayHasKey($key = 'required', $component, "{$name} has third party setting {$key}");
      $this->assertArrayHasKey($key = 'default_value', $component, "{$name} has third party setting {$key}");
    }
  }

  /**
   * Test the behavior of the form display field settings.
   */
  public function testFormDisplayFieldSettingsBehavior(): void {
    $assert_session = $this->assertSession();

    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface */
    $entity_display_repository = \Drupal::service('entity_display.repository');

    $display = $entity_display_repository->getFormDisplay('node', $this->contentType->id());
    $component = $display->getComponent('body');

    $this->drupalGet('node/add/' . $this->contentType->id());
    $assert_session->fieldExists('Body');

    $edit['title[0][value]'] = $this->randomMachineName();
    $this->submitForm($edit, 'Save');

    $node = $this->drupalGetNodeByTitle($edit['title[0][value]']);
    $this->assertNotFalse($node, 'Content was added');

    $body = $node->get('body');
    $definition = $body->getFieldDefinition();
    $default = $body->getValue();
    $default[0]['value'] = $this->randomMachineName();

    $component['third_party_settings']['form_display_field_settings']['field_type'] = $definition->getType();
    $component['third_party_settings']['form_display_field_settings']['label'] = $this->randomMachineName();
    $component['third_party_settings']['form_display_field_settings']['description'] = $this->randomMachineName();
    $component['third_party_settings']['form_display_field_settings']['required'] = FALSE;
    $component['third_party_settings']['form_display_field_settings']['default_value'] = $default;

    $display->setComponent('body', $component);
    $display->save();

    $this->drupalGet('node/add/' . $this->contentType->id());
    $field = $assert_session->fieldExists($component['third_party_settings']['form_display_field_settings']['label']);
    $assert_session->pageTextContains($component['third_party_settings']['form_display_field_settings']['description']);
    $this->assertFalse($field->hasClass('required'));
    $assert_session->fieldValueEquals('body[0][value]', $default[0]['value']);

    $component['third_party_settings']['form_display_field_settings']['required'] = TRUE;

    $display->setComponent('body', $component);
    $display->save();

    $this->drupalGet('node/add/' . $this->contentType->id());
    $this->assertTrue($field->hasClass('required'));
  }

}
