<?php

namespace Drupal\Tests\form_display_field_alter\Kernel;

use Drupal\Core\Entity\Entity\EntityFormDisplay as CoreEntityFormDisplay;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\entity_test\Entity\EntityTest;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\form_display_field_alter\Entity\EntityFormDisplay;
use Drupal\KernelTests\KernelTestBase;

/**
 * Tests the overridden entity form display.
 *
 * Copyright (C) 2025  Library Solutions, LLC (et al.).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * @covers \Drupal\form_display_field_alter\Entity\EntityFormDisplay
 * @covers \form_display_field_alter_entity_type_alter()
 * @group form_display_field_alter
 */
class EntityFormDisplayTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'entity_test',
    'field',
    'form_display_field_alter',
    'form_display_field_alter_test',
    'system',
    'user',
  ];

  /**
   * Sets up the field to use when testing entity forms.
   *
   * @return string
   *   The machine name of the field.
   */
  protected function ensureField(): string {
    $field_name = $this->randomMachineName();

    $this->installEntitySchema('field_storage_config');
    $this->installEntitySchema('field_config');
    $this->installEntitySchema('user');

    $field_storage = FieldStorageConfig::create([
      'entity_type' => 'entity_test',
      'field_name' => $field_name,
      'type' => 'string',
    ]);
    $field_storage->save();

    $field = FieldConfig::create([
      'bundle' => 'entity_test',
      'field_storage' => $field_storage,
    ]);
    $field->save();

    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface */
    $entity_display_repository_service = \Drupal::service('entity_display.repository');

    $display = $entity_display_repository_service->getFormDisplay('entity_test', 'entity_test');
    $display->setComponent($field_name, ['type' => 'form_display_field_alter_test']);
    $display->save();

    return $field_name;
  }

  /**
   * Tests that the 'entity_form_display' entity class is replaced.
   */
  public function testEntityFormDisplayEntityClass(): void {
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface */
    $entity_display_repository = \Drupal::service('entity_display.repository');

    $this->assertInstanceOf(EntityFormDisplay::class, CoreEntityFormDisplay::collectRenderDisplay(EntityTest::create([]), 'default'), 'EntityFormDisplay::collectRenderDisplay() produces the expected type');
    $this->assertInstanceOf(EntityFormDisplay::class, $entity_display_repository->getFormDisplay('entity_test', 'entity_test'), 'Entity form display storage produces the expected type');
  }

  /**
   * Tests that hook_entity_form_display_field_definition_alter() is invoked.
   */
  public function testEntityFormDisplayFieldDefinitionAlterHook(): void {
    /** @var \Drupal\Core\Entity\EntityFormBuilderInterface */
    $entity_form_builder = \Drupal::service('entity.form_builder');

    // Set up the field, and create a sample entity.
    $field_name = $this->ensureField();
    $entity = EntityTest::create([]);

    // Get the form for the sample entity.
    $form = $entity_form_builder->getForm($entity);

    /** @var \Drupal\Core\Field\FieldDefinitionInterface */
    $definition = $form[$field_name]['#definition'];
    /** @var \Drupal\Core\Field\FieldDefinitionInterface */
    $items_definition = $form[$field_name]['#items_definition'];

    $this->assertInstanceOf(FieldDefinitionInterface::class, $definition, 'Check if the widget field definition exists');
    $this->assertInstanceOf(FieldDefinitionInterface::class, $items_definition, 'Check if the field item list definition exists');
    $this->assertSame($definition, $items_definition, 'Check if the widget and field item list definitions are identical');
    $this->assertTrue($definition?->getSetting('entity_form_display_field_definition_alter'), 'Check if hook_entity_form_display_field_definition_alter() was invoked');
  }

}
